package chaincode

import (
	"github.com/hyperledger/fabric/core/chaincode/shim"
	pb "github.com/hyperledger/fabric/protos/peer"
)

// используется для смартконтрактов
type IManager interface {
	GetTxID() string
	GetTime() int64
	CreateCompositeKeyByValidObject(validObject ValidObject) (string, error)
	GetObjectByCompositeKey(expectedObject ValidObject, compositeKey string) (ValidObject, error)
	GetStateByPartialCompositeKey(objectType string, keys []string) (shim.StateQueryIteratorInterface, error)
	Save(obj ValidObject) (ValidObject, error)
	GetState(key string) ([]byte, error)
	PutState(key string, value []byte) error
	DelState(key string) error
	InvokeChaincode(chaincodeName string, args [][]byte, channel string) pb.Response
	AddObject(newObject ValidObject) (ValidObject, error)
	UpdateObject(newObject ValidObject) (ValidObject, error)
	GetObject(expectedObject ValidObject, keys []string, filterArray...string) (ValidObject, error)
	GetObjectListByKeys(expectedValidObjectType ValidObject, keys []string) ([]ValidObject, error)
	GetObjectListByKeysWithFilter(expectedValidObjectType ValidObject, keys []string, filterArray...string) ([]ValidObject, error)
	SignedInvokeChaincode(channelName string, chaincodeName string, functionName string, arg interface{}) pb.Response
	InvokeChaincode_GetUser(UserID string) pb.Response
	InvokeChaincode_VerifyUserSignature(arg interface{}) pb.Response
}

// используется внутри common
type CommonIManager interface {
	GetTime() int64
	GetStateByPartialCompositeKey(objectType string, keys []string) (shim.StateQueryIteratorInterface, error)
	CreateCompositeKey(objectType string, attributes []string) (string, error)
	CreateCompositeKeyByValidObject(validObject ValidObject) (string, error)
	GetObjectListByKeys(expectedValidObjectType ValidObject, keys []string) ([]ValidObject, error)
	GetObjectListByKeysWithFilter(expectedValidObjectType ValidObject, keys []string, filterArray...string) ([]ValidObject, error)
	SplitCompositeKey(compositeKey string) (string, []string, error)
	SetAdapter(new StubInterfaceAdapter)
	SetTimeGetter(new TimeGetter)
	GetAdapter() StubInterfaceAdapter
	GetTimeGetter() TimeGetter
	GetState(key string) ([]byte, error)
	PutState(key string, value []byte) error
	DelState(key string) error
	InvokeChaincode(chaincodeName string, args [][]byte, channel string) pb.Response
	GetTxID() string
	Save(obj ValidObject) (ValidObject, error)
	IsExists(obj ValidObject) (bool, error)
	GetIfExists(newObject ValidObject) (bool, error)
	AddObject(newObject ValidObject) (ValidObject, error)
	UpdateObject(newObject ValidObject) (ValidObject, error)
	GetObject(expectedObject ValidObject, keys []string, filterArray...string) (ValidObject, error)
	GetObjectByCompositeKey(expectedObject ValidObject, compositeKey string) (ValidObject, error)
	GetObjectByKeys(expectedObject ValidObject, keys []string) (ValidObject, error)
	GetObjectByKeysWithFilter(expectedObject ValidObject, keys []string, filterArray...string) (ValidObject, error)
	SignedInvokeChaincode(channelName string, chaincodeName string, functionName string, arg interface{}) pb.Response
	PrepareInvokeChaincodeParam(channelName string, chaincodeName string, functionName string, arg interface{}) ([][]byte, error)
	InvokeChaincode_GetUser(UserID string) pb.Response
	InvokeChaincode_VerifyUserSignature(arg interface{}) pb.Response
}
