package organizationCC

import (
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"encoding/json"
	"reflect"
)

type Organization struct {
	OrganizationID string `json:"organization_id,omitempty"`
	CompanyName    string `json:"company_name,omitempty"`
	Number         string `json:"number,omitempty"`
	LegalAddress   string `json:"legal_address,omitempty"`
	Country        string `json:"country,omitempty"`
	Website        string `json:"website,omitempty"`
}

func (this Organization) GetObjectType() string  {
	return "Organization";
}

func (this Organization) CheckBeforeAdd(manager chaincode.IManager) error {
	return nil
}

func (this Organization) New() chaincode.ValidObject {
	return &Organization{}
}

func (this Organization) CheckBeforeUpdate(updatedObject chaincode.ValidObject, manager chaincode.IManager) error {
	return nil
}

func (this Organization) GetKey(manager chaincode.IManager) []string {
	return []string{this.OrganizationID}
}

func (this Organization) Validate() error {
	if len(this.OrganizationID) == 0 {
		return errors.New("OrganizationID is empty")
	}

	return nil
}

func Unmarshal(data []byte) (*Organization, error) {
	if data == nil {
		return nil, chaincode.GlobalLogger.Error("can't Unmarshal nil data")
	}

	var obj Organization
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func Marshal(data *Organization) ([]byte, error) {
	return json.Marshal(data);
}

func Cast(validObject chaincode.ValidObject) (*Organization, error) {
	if validObject == nil {
		return nil, nil
	}

	if reflect.ValueOf(validObject).Kind() != reflect.Ptr {
		return nil, errors.New("check valid object is not reference")
	}

	casted, ok := validObject.(*Organization)
	if !ok {
		return nil, errors.New("Problem to get object")
	}

	return casted, nil;
}

