package preferenceCC

import (
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"encoding/json"
	"reflect"
	"errors"
)

type Preference struct {
	Keys  []string `json:"keys,omitempty"`
	Fields chaincode.Fields `json:"fields,omitempty"`
}

type PreferenceArray struct {
	Preference []Preference `json:"preference_array,omitempty"`
}

func UnmarshalPreferenceArray(data []byte) (*PreferenceArray, error) {
	if data == nil {
		return nil, chaincode.GlobalLogger.Error("can't Unmarshal nil data")
	}

	var obj PreferenceArray
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func (this Preference) GetObjectType() string {
	return "Preference";
}

func (this Preference) CheckBeforeAdd(manager chaincode.IManager) error {
	return nil
}

func (this Preference) CheckBeforeUpdate(updatedObject chaincode.ValidObject, manager chaincode.IManager) error {
	return nil
}

func (this Preference) New() chaincode.ValidObject {
	return &Preference{}
}

func (this Preference) Validate() error {
	errorMsg := ""

	if len(this.Keys) == 0 {
		errorMsg += "Key is empty; "
	}

	return nil
}

func (this Preference) GetKey(manager chaincode.IManager) []string {
	key := append([]string{}, this.Keys...)
	return key
}

func Unmarshal(data []byte) (*Preference, error) {
	if data == nil {
		return nil, chaincode.GlobalLogger.Error("can't Unmarshal nil data")
	}

	var obj Preference
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func Marshal(data *Preference) ([]byte, error) {
	return json.Marshal(data);
}

func Cast(validObject chaincode.ValidObject) (*Preference, error) {
	if validObject == nil {
		return nil, nil
	}

	if reflect.ValueOf(validObject).Kind() != reflect.Ptr {
		return nil, errors.New("check valid object is not reference")
	}

	casted, ok := validObject.(*Preference)
	if !ok {
		return nil, errors.New("Problem to get object")
	}

	return casted, nil;
}
