package systempreferenceCC

import (
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
)

type CryptoCurrencyConstraint struct {
	CurrencyType string `json:"currency_type"`
	MinValue     string `json:"min_value"`
	MaxValue     string `json:"max_value"`
	Scale        uint   `json:"scale"`
}

const CURRENCY_TYPE = "CURRENCY_TYPE";
const MAX_VALUE = "max_value";
const MIN_VALUE = "min_value";
const SCALE = "scale";
const IS_ENABLED = "is_enabled";

type CryptoCurrencyConstraintsList struct {
	Constraints []CryptoCurrencyConstraint `json:"constraints"`
}

func (this CryptoCurrencyConstraintsList) GetObjectType() string  {
	return "CryptoCurrencyConstraint";
}

func (this CryptoCurrencyConstraintsList) New() chaincode.ValidObject {
	return &CryptoCurrencyConstraintsList{}
}

func (this CryptoCurrencyConstraintsList) CheckBeforeAdd(manager chaincode.IManager) error {
	return nil
}

func (this CryptoCurrencyConstraintsList) CheckBeforeUpdate(updatedObject chaincode.ValidObject, manager chaincode.IManager) error {
	return nil
}

func (this CryptoCurrencyConstraintsList) GetKey(manager chaincode.IManager) []string {
	return []string{CryptoCurrenciesContrainstsKey}
}

func (this CryptoCurrencyConstraintsList) Validate() error {
	if len(this.Constraints) == 0 {
		return errors.New("Constraints is empty")
	}

	for _, v := range this.Constraints {
		if len(v.CurrencyType) == 0 {
			return errors.New("cryptoCurrencyConstraint.CurrencyType is empty")
		}
	}

	return nil
}
