package transactionCC

import (
	"encoding/json"
	"errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"reflect"
)

// история сессий хранится в канале транзакций
// о процессах которые прошли в канале transactionchannel
// о процессах которые прошли в канале bitcoinchannel
// о процессах которые прошли в канале ethereumchannel
type SessionHistory struct {
	DaemonName   string `json:"daemon_name"`
	SessionID    string `json:"session_id"`
	CurrencyType string `json:"currency_type"`
	// например TxId - Btc или Hash - Eth
	Value        string `json:"value"`
}

func (this SessionHistory) GetObjectType() string {
	return "SessionHistory";
}

func (this SessionHistory) CheckBeforeAdd(manager chaincode.IManager) error {

	return nil
}

func (this SessionHistory) CheckBeforeUpdate(updatedObject chaincode.ValidObject, manager chaincode.IManager) error {
	return errors.New("METHOD_DOES_NOT_IMPLEMENT")
}

func (this SessionHistory) GetKey(manager chaincode.IManager) []string {
	return []string{this.DaemonName, this.SessionID, this.CurrencyType, this.Value}
}

func (this SessionHistory) New() chaincode.ValidObject {
	return &SessionHistory{}
}

func (this SessionHistory) Validate() error {

	if len(this.DaemonName) == 0 {
		return errors.New("DaemonName is empty")
	}

	if len(this.SessionID) == 0 {
		return errors.New("SessionID is empty")
	}

	if len(this.CurrencyType) == 0 {
		return errors.New("CurrencyType is empty")
	}

	if len(this.Value) == 0 {
		return errors.New("Value is empty")
	}

	return nil
}

func Unmarshal_SessionHistory(data []byte) (*SessionHistory, error) {
	if data == nil {
		return nil, chaincode.GlobalLogger.Error("can't Unmarshal nil data")
	}

	var obj SessionHistory
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func Marshal_SessionHistory(data *SessionHistory) ([]byte, error) {
	return json.Marshal(data);
}

func Cast_SessionHistory(validObject chaincode.ValidObject) (*SessionHistory, error) {
	if validObject == nil {
		return nil, nil
	}

	if reflect.ValueOf(validObject).Kind() != reflect.Ptr {
		return nil, errors.New("check valid object is not reference")
	}

	casted, ok := validObject.(*SessionHistory)
	if !ok {
		return nil, errors.New("Problem to get object")
	}

	return casted, nil;
}
