package nums

import (
	"github.com/shopspring/decimal"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
)

func (this *CurrencyNumber) Add(b *CurrencyNumber) (*CurrencyNumber, error) {
	_, err := this.Validate()
	if err != nil {
		return nil, err
	}
	_, err = b.Validate()
	if err != nil {
		return nil, err
	}

	err = this.AssetEquals(b)
	if err != nil {
		return nil, err
	}

	res := decimal.NewFromFloat(0)
	res = res.Add(this.Num)
	res = res.Add(b.Num)

	result := CurrencyNumber{
		Num:   res,
		Asset: this.Asset,
	}

	return result.Validate()
}

func (this *CurrencyNumber) Sub(b *CurrencyNumber) (*CurrencyNumber, error) {
	_, err := this.Validate()
	if err != nil {
		return nil, err
	}
	_, err = b.Validate()
	if err != nil {
		return nil, err
	}

	err = this.AssetEquals(b)
	if err != nil {
		return nil, err
	}

	res := decimal.NewFromFloat(0)
	res = res.Add(this.Num)
	res = res.Sub(b.Num)

	result := CurrencyNumber{
		Num:   res,
		Asset: this.Asset,
	}

	return result.Validate()
}

func (this *CurrencyNumber) Mul(b *CurrencyNumber) (*CurrencyNumber, error) {
	_, err := this.Validate()
	if err != nil {
		return nil, err
	}
	_, err = b.Validate()
	if err != nil {
		return nil, err
	}

	err = this.AssetEquals(b)
	if err != nil {
		return nil, err
	}

	res := decimal.NewFromFloat(0)
	res = res.Add(this.Num)
	res = res.Mul(b.Num)

	result := CurrencyNumber{
		Num:   res,
		Asset: this.Asset,
	}

	return result.Validate()
}

func (this *CurrencyNumber) Div(b *CurrencyNumber) (*CurrencyNumber, error) {
	_, err := this.Validate()
	if err != nil {
		return nil, err
	}
	_, err = b.Validate()
	if err != nil {
		return nil, err
	}

	err = this.AssetEquals(b)
	if err != nil {
		return nil, err
	}

	if b.Num.Equals(decimal.Zero) {
		return nil, logging.Get().Error("dividing by zero")
	}

	res := decimal.NewFromFloat(0)
	res = res.Add(this.Num)
	// при делении 0.01 на 2 будет 0.01 если Scale = 2
	res = res.DivRound(b.Num, int32(this.Asset.Scale+1))
	// банковское округление до 0.00
	res = res.RoundBank(int32(this.Asset.Scale))

	result := CurrencyNumber{
		Num:   res,
		Asset: this.Asset,
	}

	return result.Validate()
}

func (this *CurrencyNumber) AddSelf(b *CurrencyNumber) (*CurrencyNumber, error) {
	newCurremcyNumber, err := this.Add(b)
	if err!= nil {
		return nil, err
	}
	this.Num = newCurremcyNumber.Num

	return this, nil
}

func (this *CurrencyNumber) SubSelf(b *CurrencyNumber) (*CurrencyNumber, error) {
	newCurremcyNumber, err := this.Sub(b)
	if err!= nil {
		return nil, err
	}
	this.Num = newCurremcyNumber.Num

	return this, nil
}

func (this *CurrencyNumber) MulSelf(b *CurrencyNumber) (*CurrencyNumber, error) {
	newCurremcyNumber, err := this.Mul(b)
	if err!= nil {
		return nil, err
	}
	this.Num = newCurremcyNumber.Num

	return this, nil
}

func (this *CurrencyNumber) DivSelf(b *CurrencyNumber) (*CurrencyNumber, error) {
	newCurremcyNumber, err := this.Div(b)
	if err!= nil {
		return nil, err
	}
	this.Num = newCurremcyNumber.Num

	return this, nil
}
